"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ab = require("asyncblock");
var protractor_1 = require("protractor");
var exec_1 = require("./exec");
var BrowserSync = (function () {
    function BrowserSync(_getBrowser) {
        this._getBrowser = _getBrowser;
        this.PAUSE_DEBUGGER_DELAY_MS = 500;
    }
    BrowserSync.prototype.getBrowser = function () {
        return this._getBrowser();
    };
    BrowserSync.prototype.executeScript = function (script) {
        var varArgs = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            varArgs[_i - 1] = arguments[_i];
        }
        return exec_1.exec(this.getBrowser().executeScript.apply(this.getBrowser(), arguments));
    };
    BrowserSync.prototype.executeAsyncScript = function (script) {
        var varArgs = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            varArgs[_i - 1] = arguments[_i];
        }
        return exec_1.exec(this.getBrowser().executeAsyncScript.apply(this.getBrowser(), arguments));
    };
    BrowserSync.prototype.get = function (destination, timeout) {
        return exec_1.exec(this.getBrowser().get(destination, timeout));
    };
    BrowserSync.prototype.getAllWindowHandles = function () {
        return exec_1.exec(this.getBrowser().getAllWindowHandles());
    };
    BrowserSync.prototype.getWindowHandle = function () {
        return exec_1.exec(this.getBrowser().getWindowHandle());
    };
    BrowserSync.prototype.getCurrentUrl = function () {
        return exec_1.exec(this.getBrowser().getCurrentUrl());
    };
    BrowserSync.prototype.close = function () {
        return exec_1.exec(this.getBrowser().close());
    };
    BrowserSync.prototype.quit = function () {
        return exec_1.exec(this.getBrowser().quit());
    };
    BrowserSync.prototype.actions = function () {
        return exec_1.exec(this.getBrowser().actions());
    };
    BrowserSync.prototype.switchTo = function () {
        return new TargetLocatorSync(this.getBrowser().switchTo());
    };
    BrowserSync.prototype.manage = function () {
        return new OptionsSync(this.getBrowser().manage());
    };
    BrowserSync.prototype.takeScreenshot = function () {
        return exec_1.exec(this.getBrowser().takeScreenshot());
    };
    BrowserSync.prototype.pause = function () {
        var result = this.getBrowser().pause();
        var flow = ab.getCurrentFlow();
        if (flow) {
            //Sometimes pause and debugger don't work without a delay before executing the next command
            flow.sync(setTimeout(flow.add(), this.PAUSE_DEBUGGER_DELAY_MS));
        }
        return result;
    };
    BrowserSync.prototype.debugger = function () {
        var result = this.getBrowser().debugger();
        var flow = ab.getCurrentFlow();
        if (flow) {
            //Sometimes pause and debugger don't work without a delay before executing the next command
            flow.sync(setTimeout(flow.add(), this.PAUSE_DEBUGGER_DELAY_MS));
        }
        return result;
    };
    BrowserSync.prototype.forkAndSwitchToNewDriverInstance = function (useSameUrl, copyMockModules, copyConfigUpdates) {
        var newBrowser = this.getBrowser().forkNewDriverInstance(useSameUrl, copyMockModules, copyConfigUpdates);
        var waitForAngularEnabled = this.waitForAngularEnabled();
        //Use the runner class to reset global variables after spawning the new instance
        var runner = new protractor_1.Runner(this.getProcessedConfig());
        runner.setupGlobals_(newBrowser);
        //By default the new browser will wait for angular, so we need to manually carry over the setting
        newBrowser.waitForAngularEnabled(waitForAngularEnabled);
        return this;
    };
    BrowserSync.prototype.getProcessedConfig = function () {
        return exec_1.exec(this.getBrowser().getProcessedConfig());
    };
    BrowserSync.prototype.waitForAngularEnabled = function (enabled) {
        return exec_1.exec(this.getBrowser().waitForAngularEnabled(enabled));
    };
    return BrowserSync;
}());
exports.BrowserSync = BrowserSync;
var TargetLocatorSync = (function () {
    function TargetLocatorSync(targetLocator) {
        this.targetLocator = targetLocator;
    }
    TargetLocatorSync.prototype.window = function (nameOrHandle) {
        exec_1.exec(this.targetLocator.window(nameOrHandle));
    };
    TargetLocatorSync.prototype.frame = function (frameElement) {
        exec_1.exec(this.targetLocator.frame(frameElement.getElementFinder().getWebElement()));
    };
    TargetLocatorSync.prototype.defaultContent = function () {
        exec_1.exec(this.targetLocator.defaultContent());
    };
    return TargetLocatorSync;
}());
exports.TargetLocatorSync = TargetLocatorSync;
var OptionsSync = (function () {
    function OptionsSync(options) {
        this.options = options;
    }
    OptionsSync.prototype.addCookie = function (name, value, optPath, optDomain, optIsSecure, optExpiry) {
        exec_1.exec(this.options.addCookie(name, value, optPath, optDomain, optIsSecure, optExpiry));
    };
    OptionsSync.prototype.deleteAllCookies = function () {
        exec_1.exec(this.options.deleteAllCookies());
    };
    OptionsSync.prototype.deleteCookie = function (name) {
        exec_1.exec(this.options.deleteCookie(name));
    };
    OptionsSync.prototype.getCookies = function () {
        return exec_1.exec(this.options.getCookies());
    };
    OptionsSync.prototype.getCookie = function (name) {
        return exec_1.exec(this.options.getCookie(name));
    };
    OptionsSync.prototype.window = function () {
        return new WindowSync(this.options.window());
    };
    return OptionsSync;
}());
exports.OptionsSync = OptionsSync;
var WindowSync = (function () {
    function WindowSync(window) {
        this.window = window;
    }
    WindowSync.prototype.getPosition = function () {
        return exec_1.exec(this.window.getPosition());
    };
    WindowSync.prototype.setPosition = function (x, y) {
        exec_1.exec(this.window.setPosition(x, y));
    };
    WindowSync.prototype.getSize = function () {
        return exec_1.exec(this.window.getSize());
    };
    WindowSync.prototype.setSize = function (width, height) {
        exec_1.exec(this.window.setSize(width, height));
    };
    WindowSync.prototype.maximize = function () {
        exec_1.exec(this.window.maximize());
    };
    return WindowSync;
}());
exports.WindowSync = WindowSync;
//# sourceMappingURL=browser-sync.js.map