"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
var mkdirp = require("mkdirp");
var path = require("path");
var element_finder_sync_1 = require("./element-finder-sync");
var polled_wait_1 = require("./polled-wait");
var vars_1 = require("./vars");
var DEFAULT_BREAKPOINT_WIDTH = 1366;
var DEFAULT_BREAKPOINT_HEIGHT = 1024;
/**
 * Returns the active element on the page
 */
function getActiveElement() {
    var active = vars_1.browserSync.executeScript(function () {
        return document.activeElement;
    });
    return element_finder_sync_1.ElementFinderSync.fromWebElement_(vars_1.browserSync.getBrowser(), active);
}
exports.getActiveElement = getActiveElement;
function waitForNewWindow(action, waitTimeMs) {
    var handlesBefore = vars_1.browserSync.getAllWindowHandles();
    var handles;
    action();
    waitFor(function () {
        handles = vars_1.browserSync.getAllWindowHandles();
        return handles.length === handlesBefore.length + 1;
    }, waitTimeMs);
    var newWindowHandle = handles[handles.length - 1];
    vars_1.browserSync.switchTo().window(newWindowHandle);
    waitFor(function () {
        return vars_1.browserSync.getCurrentUrl() !== '';
    }, waitTimeMs);
}
exports.waitForNewWindow = waitForNewWindow;
/**
 * Takes a screenshot and saves a .png file in the configured screenshot directory.
 *
 * @param filename The name of the file to save
 */
function takeScreenshot(filename) {
    if (filename) {
        var basePath = path.dirname(filename);
        if (!fs.existsSync(basePath)) {
            mkdirp.sync(basePath);
        }
        if (!(/\.png$/i).test(filename)) {
            filename += '.png';
        }
    }
    var base64png = vars_1.browserSync.takeScreenshot();
    if (filename) {
        fs.writeFileSync(filename, base64png, 'base64');
    }
    return base64png;
}
exports.takeScreenshot = takeScreenshot;
function calculateDimension(dimension, window, viewport) {
    return dimension + (window - viewport);
}
function resizeViewport(size) {
    var windowSize = vars_1.browserSync.manage().window().getSize();
    var viewportSize = vars_1.browserSync.executeScript(function () {
        return {
            height: window.document.documentElement.clientHeight,
            width: window.document.documentElement.clientWidth
        };
    });
    var calcWidth = function (width) { return calculateDimension(width, windowSize.width, viewportSize.width); };
    var calcHeight = function (height) { return calculateDimension(height, windowSize.height, viewportSize.height); };
    var width = windowSize.width;
    var height = windowSize.height;
    if (size) {
        width = calcWidth(size.width || DEFAULT_BREAKPOINT_WIDTH);
        height = calcHeight(size.height || DEFAULT_BREAKPOINT_HEIGHT);
    }
    else if (windowSize.width < DEFAULT_BREAKPOINT_WIDTH) {
        width = calcWidth(DEFAULT_BREAKPOINT_WIDTH);
    }
    else {
        // No size set and width is wider than the minimum.  We can return early without resizing the browser
        return;
    }
    vars_1.browserSync.manage().window().setSize(width, height);
}
exports.resizeViewport = resizeViewport;
exports.disallowExpect = (function () {
    var ALLOWED_LOCATIONS = ['node_modules', 'protractor_sync.'];
    function disableMethod(object, method, extraInfo) {
        if (object[method] == null) {
            throw new Error('Cannot disable ' + method + '(). It does not exist');
        }
        var original = object[method];
        object[method] = function () {
            // We don't want to block access from protractor or selenium or the current file
            var stack = new Error().stack;
            //First line is the error message, second line is where the error was created, third line is the caller
            var secondFrame = stack.split('\n')[2];
            if (ALLOWED_LOCATIONS.every(function (location) { return secondFrame.indexOf(location) < 0; })) {
                throw new Error(method + '() has been disabled in this project! ' + (extraInfo || ''));
            }
            else {
                return original.apply(this, arguments);
            }
        };
    }
    return function () {
        var EXPECT_ADVICE = 'Use polledExpect instead of expect.';
        disableMethod(global, 'expect', EXPECT_ADVICE);
    };
})();
function waitFor(condition, waitTimeMs) {
    polled_wait_1.polledWait(function () {
        return { data: null, keepPolling: !condition() };
    }, null, waitTimeMs);
}
exports.waitFor = waitFor;
//# sourceMappingURL=utility.js.map